@@ -1,15 +1,21 @@
 #!/bin/sh
-# $Id: testminissdpdnotif.sh,v 1.3 2016/12/16 09:22:36 nanard Exp $
-# (c) 2016 Thomas Bernard
+# $Id: testminissdpdnotif.sh,v 1.4 2019/02/10 13:39:18 nanard Exp $
+# (c) 2019 Thomas Bernard
 
 OS=`uname -s`
-IF=lo
-if [ "$OS" = "Darwin" ] || [ "$OS" = "SunOS" ] ; then
-	IF=lo0
-fi
+
 # if set, 1st argument is network interface
 if [ -n "$1" ] ; then
 	IF=$1
+else
+	case $OS in
+		*BSD | Darwin | SunOS)
+			IF=lo0
+			;;
+		*)
+			IF=lo
+			;;
+	esac
 fi
 
 # trap sigint in the script so CTRL-C interrupts the running program,
